@@ -397,7 +397,7 @@ void Town::disp_main_menu(int refreshFlag)
 
 		if( button_grant.init_flag )
 		{
-			if( nation_array[nation_recno]->cash > 0 )
+			if( nation_recno && nation_array[nation_recno]->cash > 0 )
 				button_grant.enable();
 			else
 				button_grant.disable();
@@ -552,10 +552,10 @@ void Town::detect_main_menu()
 
 	//------- detect buttons --------//
 
-	if( button_recruit.detect('R') )
+	if( button_recruit.detect(GETKEY(KEYEVENT_TOWN_RECRUIT)) )
 		recruit(-1, 0, COMMAND_PLAYER);
 
-	if( button_train.detect('B') )
+	if( button_train.detect(GETKEY(KEYEVENT_TOWN_TRAIN)) )
 	{
 		town_menu_mode = TOWN_MENU_TRAIN;
 		disable_refresh = 1;    // static var for disp_info() only
