@@ -108,18 +108,18 @@ static short build_firm_button_order[MAX_FIRM_TYPE] =
 };
 // ##### end Gilbert 3/10 #######//
 
-static char button_build_hotkey[MAX_FIRM_TYPE] =
+static KeyEventType button_build_hotkey[MAX_FIRM_TYPE] =
 {
-	'F', // fort
-	'R', // mine (raw)
-	'A', // factory
-	'M', // market
-	'T', // tower of science
-	'W', // war factory
-	'I', // inn
-	'H', // harbour
-	'P', // seat of power
-	 0, // monster
+	KEYEVENT_BUILD_CAMP,
+	KEYEVENT_BUILD_MINE,
+	KEYEVENT_BUILD_FACTORY,
+	KEYEVENT_BUILD_MARKET,
+	KEYEVENT_BUILD_RESEARCH,
+	KEYEVENT_BUILD_WAR_FACTORY,
+	KEYEVENT_BUILD_INN,
+	KEYEVENT_BUILD_HARBOR,
+	KEYEVENT_BUILD_BASE,
+	KEYEVENT_BUILD_MONSTER,
 };
 
 
@@ -583,7 +583,7 @@ void Unit::detect_button()
 
 	//--------- "return camp" button ---------//
 
-	if( home_camp_firm_recno && button_return_camp.detect('R') )
+	if( home_camp_firm_recno && button_return_camp.detect(GETKEY(KEYEVENT_UNIT_RETURN)) )
 	{
 		// sound effect
 		se_res.far_sound(next_x_loc(), next_y_loc(), 1, 'S', sprite_id, "ACK");
@@ -601,7 +601,7 @@ void Unit::detect_button()
 
 	//-------- build button --------//
 
-	if( button_build.detect('B') )
+	if( button_build.detect(GETKEY(KEYEVENT_UNIT_BUILD)) )
 	{
 		unit_menu_mode = UNIT_MENU_BUILD;
 		info.disp();
@@ -609,7 +609,7 @@ void Unit::detect_button()
 
 	//-------- settle button ---------//
 
-	if( button_settle.detect('T') )
+	if( button_settle.detect(GETKEY(KEYEVENT_UNIT_SETTLE)) )
 	{
 		power.issue_command(COMMAND_SETTLE, sprite_recno);
 		info.disp();
@@ -883,6 +883,8 @@ static void group_drop_spy_identity()
 			if( !remote.is_enable() )
 			{
 				spy_array[unitPtr->spy_recno]->drop_spy_identity();
+				if( unitPtr->sprite_recno == unit_array.selected_recno )
+					info.disp();
 			}
 			else
 			{
@@ -967,7 +969,7 @@ void Unit::detect_build_menu()
 
 		if( button_build_flag[i] && firm_res[firmId]->can_build(sprite_recno) )
 		{
-			if( button_build_array[i].detect(button_build_hotkey[i]) )
+			if( button_build_array[i].detect(GETKEY(button_build_hotkey[i])) )
 			{
 				power.issue_command(COMMAND_BUILD_FIRM, sprite_recno, firmId);
 				rc = 1;
@@ -998,6 +1000,20 @@ void Unit::detect_build_menu()
 //----------- End of function Unit::detect_build_menu -----------//
 
 
+const char *select_where_to_build[MAX_FIRM_TYPE] =
+{
+	// TRANSLATORS: Please select a location to build the <Firm>.
+	N_("Please select a location to build the Seat of Power."),
+	N_("Please select a location to build the Factory."),
+	N_("Please select a location to build the Inn."),
+	N_("Please select a location to build the Market."),
+	N_("Please select a location to build the Fort."),
+	N_("Please select a location to build the Mine."),
+	N_("Please select a location to build the Tower of Science."),
+	N_("Please select a location to build the War Factory."),
+	N_("Please select a location to build the Harbor."),
+	("Please select a location to build the Fryhtan Lair."),
+};
 //--------- Begin of function Unit::disp_build ---------//
 //
 // Display the info when the player has selected the type of
@@ -1011,8 +1027,7 @@ void Unit::disp_build(int refreshFlag)
 
 		String str;
 
-		// TRANSLATORS: Please select a location to build the <Firm>.
-		snprintf( str, MAX_STR_LEN+1, _("Please select a location to build the %s."), _(firm_res[power.command_para]->name) );
+		str = _(select_where_to_build[power.command_para-1]);
 
 // FRENCH
 //		font_san.put_paragraph( INFO_X1, INFO_Y1, INFO_X2, INFO_Y2, str, 0 );
