@@ -390,7 +390,7 @@ void Vga::handle_messages()
             case SDL_WINDOWEVENT_FOCUS_GAINED:
             case SDL_WINDOWEVENT_RESTORED:
                sys.need_redraw_flag = 1;
-               if( !sys.is_mp_game )
+               if( !sys.is_mp_game && config_adv.vga_pause_on_focus_loss )
                   sys.unpause();
 
                // update ctrl/shift/alt key state
@@ -401,7 +401,7 @@ void Vga::handle_messages()
             //case SDL_WINDOWEVENT_LEAVE: // Do not respond to mouse focus
             case SDL_WINDOWEVENT_FOCUS_LOST:
             case SDL_WINDOWEVENT_MINIMIZED:
-               if( !sys.is_mp_game )
+               if( !sys.is_mp_game && config_adv.vga_pause_on_focus_loss )
                   sys.pause();
                // turn the system cursor back on to get around a fullscreen
                // mouse grabbed problem on windows
@@ -532,6 +532,8 @@ void Vga::handle_messages()
                   set_mouse_mode( MOUSE_INPUT_ABS );
             }
          }
+         if( SDL_IsTextInputActive() && event.key.keysym.sym >= SDLK_SPACE && event.key.keysym.sym <= SDLK_z )
+		bypass = 1;
          if( !bypass )
          {
             mouse.update_skey_state();
@@ -543,6 +545,9 @@ void Vga::handle_messages()
          mouse.update_skey_state();
          break;
       case SDL_TEXTINPUT:
+         mouse.add_typing_event(event.text.text, misc.get_time());
+         break;
+      case SDL_TEXTEDITING:
       case SDL_JOYAXISMOTION:
       case SDL_JOYBALLMOTION:
       case SDL_JOYHATMOTION:
@@ -824,6 +829,9 @@ void Vga::save_status_report()
       fprintf(file, "Big endian\n");
    else
       fprintf(file, "Little endian\n");
+#ifndef HAVE_KNOWN_BUILD
+   fprintf(file, "Binary built using unsupported configuration\n");
+#endif
 
    s = SDL_GetCurrentVideoDriver();
    fprintf(file, "Current SDL video driver: %s\n", s);
